package sun.java2d.pipe;

import java.awt.Rectangle;
import java.awt.geom.RectangularShape;
import java.awt.geom.AffineTransform;
import java.awt.Shape;

public class Region
{
    static final int INIT_SIZE = 50;
    static final int GROW_SIZE = 50;
    public static final Region EMPTY_REGION;
    public static final Region WHOLE_REGION;
    int lox;
    int loy;
    int hix;
    int hiy;
    int endIndex;
    int[] bands;
    static final int INCLUDE_A = 1;
    static final int INCLUDE_B = 2;
    static final int INCLUDE_COMMON = 4;
    
    private static native void initIDs();
    
    public static int dimAdd(int n, int n2) {
        if (n2 <= 0) {
            return n;
        }
        if ((n2 += n) < n) {
            return Integer.MAX_VALUE;
        }
        return n2;
    }
    
    public static int clipAdd(int n, int n2) {
        int n3 = n + n2;
        if (n3 > n != n2 > 0) {
            n3 = ((n2 < 0) ? Integer.MIN_VALUE : Integer.MAX_VALUE);
        }
        return n3;
    }
    
    public static int clipScale(int n, double n2) {
        if (n2 == 1.0) {
            return n;
        }
        double a = (double)n * n2;
        if (a < -2.147483648E9) {
            return Integer.MIN_VALUE;
        }
        if (a > 2.147483647E9) {
            return Integer.MAX_VALUE;
        }
        return (int)Math.round(a);
    }
    
    protected Region(int lox, int loy, int hix, int hiy) {
        this.lox = lox;
        this.loy = loy;
        this.hix = hix;
        this.hiy = hiy;
    }
    
    private Region(int lox, int loy, int hix, int hiy, int[] bands, int endIndex) {
        this.lox = lox;
        this.loy = loy;
        this.hix = hix;
        this.hiy = hiy;
        this.bands = bands;
        this.endIndex = endIndex;
    }
    
    public static Region getInstance(Shape shape, AffineTransform affineTransform) {
        return getInstance(Region.WHOLE_REGION, false, shape, affineTransform);
    }
    
    public static Region getInstance(Region region, Shape shape, AffineTransform affineTransform) {
        return getInstance(region, false, shape, affineTransform);
    }
    
    public static Region getInstance(Region outputArea, boolean b, Shape shape, AffineTransform affineTransform) {
        if (shape instanceof RectangularShape && ((RectangularShape)shape).isEmpty()) {
            return Region.EMPTY_REGION;
        }
        int[] array = new int[4];
        ShapeSpanIterator shapeSpanIterator = new ShapeSpanIterator(b);
        try {
            shapeSpanIterator.setOutputArea(outputArea);
            shapeSpanIterator.appendPath(shape.getPathIterator(affineTransform));
            shapeSpanIterator.getPathBox(array);
            Region instance = getInstance(array);
            instance.appendSpans(shapeSpanIterator);
            return instance;
        }
        finally {
            shapeSpanIterator.dispose();
        }
    }
    
    static Region getInstance(int b, int b2, int b3, int b4, int[] array) {
        int i = array[0];
        int n = array[1];
        if (b4 <= b2 || b3 <= b || n <= i) {
            return Region.EMPTY_REGION;
        }
        int[] array2 = new int[(n - i) * 5];
        int n2 = 0;
        int n3 = 2;
        for (; i < n; ++i) {
            int max = Math.max(clipAdd(b, array[n3++]), b);
            int min = Math.min(clipAdd(b, array[n3++]), b3);
            if (max < min) {
                int max2 = Math.max(clipAdd(b2, i), b2);
                int min2 = Math.min(clipAdd(max2, 1), b4);
                if (max2 < min2) {
                    array2[n2++] = max2;
                    array2[n2++] = min2;
                    array2[n2++] = 1;
                    array2[n2++] = max;
                    array2[n2++] = min;
                }
            }
        }
        return (n2 != 0) ? new Region(b, b2, b3, b4, array2, n2) : Region.EMPTY_REGION;
    }
    
    public static Region getInstance(Rectangle rectangle) {
        return getInstanceXYWH(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    public static Region getInstanceXYWH(int n, int n2, int n3, int n4) {
        return getInstanceXYXY(n, n2, dimAdd(n, n3), dimAdd(n2, n4));
    }
    
    public static Region getInstance(int[] array) {
        return new Region(array[0], array[1], array[2], array[3]);
    }
    
    public static Region getInstanceXYXY(int n, int n2, int n3, int n4) {
        return new Region(n, n2, n3, n4);
    }
    
    public void setOutputArea(Rectangle rectangle) {
        this.setOutputAreaXYWH(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    public void setOutputAreaXYWH(int n, int n2, int n3, int n4) {
        this.setOutputAreaXYXY(n, n2, dimAdd(n, n3), dimAdd(n2, n4));
    }
    
    public void setOutputArea(int[] array) {
        this.lox = array[0];
        this.loy = array[1];
        this.hix = array[2];
        this.hiy = array[3];
    }
    
    public void setOutputAreaXYXY(int lox, int loy, int hix, int hiy) {
        this.lox = lox;
        this.loy = loy;
        this.hix = hix;
        this.hiy = hiy;
    }
    
    public void appendSpans(SpanIterator spanIterator) {
        int[] array = new int[6];
        while (spanIterator.nextSpan(array)) {
            this.appendSpan(array);
        }
        this.endRow(array);
        this.calcBBox();
    }
    
    public Region getScaledRegion(double n, double n2) {
        if (n == 0.0 || n2 == 0.0 || this == Region.EMPTY_REGION) {
            return Region.EMPTY_REGION;
        }
        if ((n == 1.0 && n2 == 1.0) || this == Region.WHOLE_REGION) {
            return this;
        }
        Region region = new Region(clipScale(this.lox, n), clipScale(this.loy, n2), clipScale(this.hix, n), clipScale(this.hiy, n2));
        int[] bands = this.bands;
        if (bands != null) {
            int endIndex = this.endIndex;
            int[] bands2 = new int[endIndex];
            int i = 0;
            int endIndex2 = 0;
            while (i < endIndex) {
                int n3 = bands2[endIndex2++] = clipScale(bands[i++], n2);
                int n4 = bands2[endIndex2++] = clipScale(bands[i++], n2);
                int n5 = bands2[endIndex2++] = bands[i++];
                if (n3 < n4) {
                    while (--n5 >= 0) {
                        int clipScale = clipScale(bands[i++], n);
                        int clipScale2 = clipScale(bands[i++], n);
                        if (clipScale < clipScale2) {
                            bands2[endIndex2++] = clipScale;
                            bands2[endIndex2++] = clipScale2;
                        }
                        else {
                            continue;
                        }
                    }
                }
                else {
                    i += n5 * 2;
                }
                if (endIndex2 > endIndex2) {
                    bands2[endIndex2 - 1] = (endIndex2 - endIndex2) / 2;
                }
                else {
                    endIndex2 -= 3;
                }
            }
            if (endIndex2 <= 5) {
                if (endIndex2 < 5) {
                    Region region2 = region;
                    int n6 = 0;
                    region.hiy = n6;
                    region.hix = n6;
                    region.loy = n6;
                    region2.lox = n6;
                }
                else {
                    region.loy = bands2[0];
                    region.hiy = bands2[1];
                    region.lox = bands2[3];
                    region.hix = bands2[4];
                }
            }
            else {
                region.endIndex = endIndex2;
                region.bands = bands2;
            }
        }
        return region;
    }
    
    public Region getTranslatedRegion(int n, int n2) {
        if ((n | n2) == 0x0) {
            return this;
        }
        int n3 = this.lox + n;
        int n4 = this.loy + n2;
        int n5 = this.hix + n;
        int n6 = this.hiy + n2;
        if (n3 > this.lox != n > 0 || n4 > this.loy != n2 > 0 || n5 > this.hix != n > 0 || n6 > this.hiy != n2 > 0) {
            return this.getSafeTranslatedRegion(n, n2);
        }
        Region region = new Region(n3, n4, n5, n6);
        int[] bands = this.bands;
        if (bands != null) {
            int endIndex = this.endIndex;
            region.endIndex = endIndex;
            int[] bands2 = new int[endIndex];
            region.bands = bands2;
            int i = 0;
            while (i < endIndex) {
                bands2[i] = bands[i] + n2;
                ++i;
                bands2[i] = bands[i] + n2;
                ++i;
                int n7 = bands2[i] = bands[i];
                ++i;
                while (--n7 >= 0) {
                    bands2[i] = bands[i] + n;
                    ++i;
                    bands2[i] = bands[i] + n;
                    ++i;
                }
            }
            return region;
        }
        return region;
    }
    
    private Region getSafeTranslatedRegion(int n, int n2) {
        Region region = new Region(clipAdd(this.lox, n), clipAdd(this.loy, n2), clipAdd(this.hix, n), clipAdd(this.hiy, n2));
        int[] bands = this.bands;
        if (bands != null) {
            int endIndex = this.endIndex;
            int[] bands2 = new int[endIndex];
            int i = 0;
            int endIndex2 = 0;
            while (i < endIndex) {
                int n3 = bands2[endIndex2++] = clipAdd(bands[i++], n2);
                int n4 = bands2[endIndex2++] = clipAdd(bands[i++], n2);
                int n5 = bands2[endIndex2++] = bands[i++];
                if (n3 < n4) {
                    while (--n5 >= 0) {
                        int clipAdd = clipAdd(bands[i++], n);
                        int clipAdd2 = clipAdd(bands[i++], n);
                        if (clipAdd < clipAdd2) {
                            bands2[endIndex2++] = clipAdd;
                            bands2[endIndex2++] = clipAdd2;
                        }
                        else {
                            continue;
                        }
                    }
                }
                else {
                    i += n5 * 2;
                }
                if (endIndex2 > endIndex2) {
                    bands2[endIndex2 - 1] = (endIndex2 - endIndex2) / 2;
                }
                else {
                    endIndex2 -= 3;
                }
            }
            if (endIndex2 <= 5) {
                if (endIndex2 < 5) {
                    Region region2 = region;
                    int n6 = 0;
                    region.hiy = n6;
                    region.hix = n6;
                    region.loy = n6;
                    region2.lox = n6;
                }
                else {
                    region.loy = bands2[0];
                    region.hiy = bands2[1];
                    region.lox = bands2[3];
                    region.hix = bands2[4];
                }
            }
            else {
                region.endIndex = endIndex2;
                region.bands = bands2;
            }
        }
        return region;
    }
    
    public Region getIntersection(Rectangle rectangle) {
        return this.getIntersectionXYWH(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    public Region getIntersectionXYWH(int n, int n2, int n3, int n4) {
        return this.getIntersectionXYXY(n, n2, dimAdd(n, n3), dimAdd(n2, n4));
    }
    
    public Region getIntersectionXYXY(int n, int n2, int n3, int n4) {
        if (this.isInsideXYXY(n, n2, n3, n4)) {
            return this;
        }
        Region o = new Region((n < this.lox) ? this.lox : n, (n2 < this.loy) ? this.loy : n2, (n3 > this.hix) ? this.hix : n3, (n4 > this.hiy) ? this.hiy : n4);
        if (this.bands != null) {
            o.appendSpans(this.getSpanIterator());
            return o;
        }
        return o;
    }
    
    public Region getIntersection(Region region) {
        if (this.isInsideQuickCheck(region)) {
            return this;
        }
        if (region.isInsideQuickCheck(this)) {
            return region;
        }
        Region o = new Region((region.lox < this.lox) ? this.lox : region.lox, (region.loy < this.loy) ? this.loy : region.loy, (region.hix > this.hix) ? this.hix : region.hix, (region.hiy > this.hiy) ? this.hiy : region.hiy);
        if (!o.isEmpty()) {
            o.filterSpans(this, region, 4);
            return o;
        }
        return o;
    }
    
    public Region getUnion(Region region) {
        if (region.isEmpty() || region.isInsideQuickCheck(this)) {
            return this;
        }
        if (this.isEmpty() || this.isInsideQuickCheck(region)) {
            return region;
        }
        Region o = new Region((region.lox > this.lox) ? this.lox : region.lox, (region.loy > this.loy) ? this.loy : region.loy, (region.hix < this.hix) ? this.hix : region.hix, (region.hiy < this.hiy) ? this.hiy : region.hiy);
        o.filterSpans(this, region, 7);
        return o;
    }
    
    public Region getDifference(Region region) {
        if (!region.intersectsQuickCheck(this)) {
            return this;
        }
        if (this.isInsideQuickCheck(region)) {
            return Region.EMPTY_REGION;
        }
        Region region2 = new Region(this.lox, this.loy, this.hix, this.hiy);
        region2.filterSpans(this, region, 1);
        return region2;
    }
    
    public Region getExclusiveOr(Region region) {
        if (region.isEmpty()) {
            return this;
        }
        if (this.isEmpty()) {
            return region;
        }
        Region o = new Region((region.lox > this.lox) ? this.lox : region.lox, (region.loy > this.loy) ? this.loy : region.loy, (region.hix < this.hix) ? this.hix : region.hix, (region.hiy < this.hiy) ? this.hiy : region.hiy);
        o.filterSpans(this, region, 3);
        return o;
    }
    
    private void filterSpans(Region region, Region region2, int n) {
        int[] bands = region.bands;
        int[] bands2 = region2.bands;
        if (bands == null) {
            bands = new int[] { region.loy, region.hiy, 1, region.lox, region.hix };
        }
        if (bands2 == null) {
            bands2 = new int[] { region2.loy, region2.hiy, 1, region2.lox, region2.hix };
        }
        int[] array = new int[6];
        int i = 0;
        int hiy = bands[i++];
        int n2 = bands[i++];
        int n3 = i + 2 * bands[i++];
        int j = 0;
        int hiy2 = bands2[j++];
        int n4 = bands2[j++];
        int n5 = j + 2 * bands2[j++];
        int k = this.loy;
        while (k < this.hiy) {
            if (k >= n2) {
                if (n3 < region.endIndex) {
                    i = n3;
                    hiy = bands[i++];
                    n2 = bands[i++];
                    n3 = i + 2 * bands[i++];
                }
                else {
                    if ((n & 0x2) == 0x0) {
                        break;
                    }
                    n2 = (hiy = this.hiy);
                }
            }
            else if (k >= n4) {
                if (n5 < region2.endIndex) {
                    j = n5;
                    hiy2 = bands2[j++];
                    n4 = bands2[j++];
                    n5 = j + 2 * bands2[j++];
                }
                else {
                    if ((n & 0x1) == 0x0) {
                        break;
                    }
                    n4 = (hiy2 = this.hiy);
                }
            }
            else {
                int n6;
                if (k < hiy2) {
                    if (k < hiy) {
                        k = Math.min(hiy, hiy2);
                        continue;
                    }
                    n6 = Math.min(n2, hiy2);
                    if ((n & 0x1) != 0x0) {
                        array[1] = k;
                        array[3] = n6;
                        while (i < n3) {
                            array[0] = bands[i++];
                            array[2] = bands[i++];
                            this.appendSpan(array);
                        }
                    }
                }
                else if (k < hiy) {
                    n6 = Math.min(n4, hiy);
                    if ((n & 0x2) != 0x0) {
                        array[1] = k;
                        array[3] = n6;
                        while (j < n5) {
                            array[0] = bands2[j++];
                            array[2] = bands2[j++];
                            this.appendSpan(array);
                        }
                    }
                }
                else {
                    n6 = Math.min(n2, n4);
                    array[1] = k;
                    array[3] = n6;
                    int hix = bands[i++];
                    int n7 = bands[i++];
                    int hix2 = bands2[j++];
                    int n8 = bands2[j++];
                    int l = Math.min(hix, hix2);
                    if (l < this.lox) {
                        l = this.lox;
                    }
                    while (l < this.hix) {
                        if (l >= n7) {
                            if (i < n3) {
                                hix = bands[i++];
                                n7 = bands[i++];
                            }
                            else {
                                if ((n & 0x2) == 0x0) {
                                    break;
                                }
                                n7 = (hix = this.hix);
                            }
                        }
                        else if (l >= n8) {
                            if (j < n5) {
                                hix2 = bands2[j++];
                                n8 = bands2[j++];
                            }
                            else {
                                if ((n & 0x1) == 0x0) {
                                    break;
                                }
                                n8 = (hix2 = this.hix);
                            }
                        }
                        else {
                            int n9;
                            int n10;
                            if (l < hix2) {
                                if (l < hix) {
                                    n9 = Math.min(hix, hix2);
                                    n10 = 0;
                                }
                                else {
                                    n9 = Math.min(n7, hix2);
                                    n10 = ((n & 0x1) != 0x0);
                                }
                            }
                            else if (l < hix) {
                                n9 = Math.min(hix, n8);
                                n10 = ((n & 0x2) != 0x0);
                            }
                            else {
                                n9 = Math.min(n7, n8);
                                n10 = ((n & 0x4) != 0x0);
                            }
                            if (n10 != 0) {
                                array[0] = l;
                                array[2] = n9;
                                this.appendSpan(array);
                            }
                            l = n9;
                        }
                    }
                }
                k = n6;
            }
        }
        this.endRow(array);
        this.calcBBox();
    }
    
    public Region getBoundsIntersection(Rectangle rectangle) {
        return this.getBoundsIntersectionXYWH(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    public Region getBoundsIntersectionXYWH(int n, int n2, int n3, int n4) {
        return this.getBoundsIntersectionXYXY(n, n2, dimAdd(n, n3), dimAdd(n2, n4));
    }
    
    public Region getBoundsIntersectionXYXY(int n, int n2, int n3, int n4) {
        if (this.bands == null && this.lox >= n && this.loy >= n2 && this.hix <= n3 && this.hiy <= n4) {
            return this;
        }
        return new Region((n < this.lox) ? this.lox : n, (n2 < this.loy) ? this.loy : n2, (n3 > this.hix) ? this.hix : n3, (n4 > this.hiy) ? this.hiy : n4);
    }
    
    public Region getBoundsIntersection(Region region) {
        if (this.encompasses(region)) {
            return region;
        }
        if (region.encompasses(this)) {
            return this;
        }
        return new Region((region.lox < this.lox) ? this.lox : region.lox, (region.loy < this.loy) ? this.loy : region.loy, (region.hix > this.hix) ? this.hix : region.hix, (region.hiy > this.hiy) ? this.hiy : region.hiy);
    }
    
    private void appendSpan(int[] array) {
        int lox;
        if ((lox = array[0]) < this.lox) {
            lox = this.lox;
        }
        int loy;
        if ((loy = array[1]) < this.loy) {
            loy = this.loy;
        }
        int hix;
        if ((hix = array[2]) > this.hix) {
            hix = this.hix;
        }
        int hiy;
        if ((hiy = array[3]) > this.hiy) {
            hiy = this.hiy;
        }
        if (hix <= lox || hiy <= loy) {
            return;
        }
        int n = array[4];
        if (this.endIndex == 0 || loy >= this.bands[n + 1]) {
            if (this.bands == null) {
                this.bands = new int[50];
            }
            else {
                this.needSpace(5);
                this.endRow(array);
                n = array[4];
            }
            this.bands[this.endIndex++] = loy;
            this.bands[this.endIndex++] = hiy;
            this.bands[this.endIndex++] = 0;
        }
        else if (loy == this.bands[n] && hiy == this.bands[n + 1] && lox >= this.bands[this.endIndex - 1]) {
            if (lox == this.bands[this.endIndex - 1]) {
                this.bands[this.endIndex - 1] = hix;
                return;
            }
            this.needSpace(2);
        }
        else {
            throw new InternalError("bad span");
        }
        this.bands[this.endIndex++] = lox;
        this.bands[this.endIndex++] = hix;
        int[] bands = this.bands;
        int n2 = n + 2;
        ++bands[n2];
        return;
    }
    
    private void needSpace(int n) {
        if (this.endIndex + n >= this.bands.length) {
            int[] bands = new int[this.bands.length + 50];
            System.arraycopy(this.bands, 0, bands, 0, this.endIndex);
            this.bands = bands;
        }
    }
    
    private void endRow(int[] array) {
        int n = array[4];
        int endIndex = array[5];
        if (n > endIndex) {
            int[] bands = this.bands;
            if (bands[endIndex + 1] == bands[n] && bands[endIndex + 2] == bands[n + 2]) {
                int n2;
                for (n2 = bands[n + 2] * 2, n += 3, endIndex += 3; n2 > 0 && bands[n++] == bands[endIndex++]; --n2) {}
                if (n2 == 0) {
                    bands[array[5] + 1] = bands[endIndex + 1];
                    this.endIndex = endIndex;
                    return;
                }
            }
        }
        array[5] = array[4];
        array[4] = this.endIndex;
    }
    
    private void calcBBox() {
        int[] bands = this.bands;
        if (this.endIndex <= 5) {
            if (this.endIndex == 0) {
                int n3 = 0;
                this.hiy = n3;
                this.hix = n3;
                this.loy = n3;
                this.lox = n3;
            }
            else {
                this.loy = bands[0];
                this.hiy = bands[1];
                this.lox = bands[3];
                this.hix = bands[4];
                this.endIndex = 0;
            }
            this.bands = null;
            return;
        }
        else {
            int hix = this.hix;
            int lox = this.lox;
            int n = 0;
            int i = 0;
            while (i < this.endIndex) {
                n = i;
                int n2 = bands[i + 2];
                i += 3;
                if (hix > bands[i]) {
                    hix = bands[i];
                }
                i += n2 * 2;
                if (lox < bands[i - 1]) {
                    lox = bands[i - 1];
                }
                else {
                    continue;
                }
            }
            this.lox = hix;
            this.loy = bands[0];
            this.hix = lox;
            this.hiy = bands[n + 1];
            return;
        }
    }
    
    public final int getLoX() {
        return this.lox;
    }
    
    public final int getLoY() {
        return this.loy;
    }
    
    public final int getHiX() {
        return this.hix;
    }
    
    public final int getHiY() {
        return this.hiy;
    }
    
    public final int getWidth() {
        if (this.hix < this.lox) {
            return 0;
        }
        int n;
        if ((n = this.hix - this.lox) < 0) {
            n = Integer.MAX_VALUE;
        }
        return n;
    }
    
    public final int getHeight() {
        if (this.hiy < this.loy) {
            return 0;
        }
        int n;
        if ((n = this.hiy - this.loy) < 0) {
            n = Integer.MAX_VALUE;
        }
        return n;
    }
    
    public boolean isEmpty() {
        return this.hix <= this.lox || this.hiy <= this.loy;
    }
    
    public boolean isRectangular() {
        return this.bands == null;
    }
    
    public boolean contains(int n, int n2) {
        if (n < this.lox || n >= this.hix || n2 < this.loy || n2 >= this.hiy) {
            return false;
        }
        if (this.bands == null) {
            return true;
        }
        int i = 0;
        while (i < this.endIndex) {
            if (n2 < this.bands[i++]) {
                return false;
            }
            if (n2 >= this.bands[i++]) {
                i += this.bands[i++] * 2;
            }
            else {
                while (i < i + this.bands[i++] * 2) {
                    if (n < this.bands[i++]) {
                        return false;
                    }
                    if (n >= this.bands[i++]) {
                        continue;
                    }
                    return true;
                }
                return false;
            }
        }
        return false;
    }
    
    public boolean isInsideXYWH(int n, int n2, int n3, int n4) {
        return this.isInsideXYXY(n, n2, dimAdd(n, n3), dimAdd(n2, n4));
    }
    
    public boolean isInsideXYXY(int n, int n2, int n3, int n4) {
        return this.lox >= n && this.loy >= n2 && this.hix <= n3 && this.hiy <= n4;
    }
    
    public boolean isInsideQuickCheck(Region region) {
        return region.bands == null && region.lox <= this.lox && region.loy <= this.loy && region.hix >= this.hix && region.hiy >= this.hiy;
    }
    
    public boolean intersectsQuickCheckXYXY(int n, int n2, int n3, int n4) {
        return n3 > this.lox && n < this.hix && n4 > this.loy && n2 < this.hiy;
    }
    
    public boolean intersectsQuickCheck(Region region) {
        return region.hix > this.lox && region.lox < this.hix && region.hiy > this.loy && region.loy < this.hiy;
    }
    
    public boolean encompasses(Region region) {
        return this.bands == null && this.lox <= region.lox && this.loy <= region.loy && this.hix >= region.hix && this.hiy >= region.hiy;
    }
    
    public boolean encompassesXYWH(int n, int n2, int n3, int n4) {
        return this.encompassesXYXY(n, n2, dimAdd(n, n3), dimAdd(n2, n4));
    }
    
    public boolean encompassesXYXY(int n, int n2, int n3, int n4) {
        return this.bands == null && this.lox <= n && this.loy <= n2 && this.hix >= n3 && this.hiy >= n4;
    }
    
    public void getBounds(int[] array) {
        array[0] = this.lox;
        array[1] = this.loy;
        array[2] = this.hix;
        array[3] = this.hiy;
    }
    
    public void clipBoxToBounds(int[] array) {
        if (array[0] < this.lox) {
            array[0] = this.lox;
        }
        if (array[1] < this.loy) {
            array[1] = this.loy;
        }
        if (array[2] > this.hix) {
            array[2] = this.hix;
        }
        if (array[3] > this.hiy) {
            array[3] = this.hiy;
        }
    }
    
    public RegionIterator getIterator() {
        return new RegionIterator(this);
    }
    
    public SpanIterator getSpanIterator() {
        return new RegionSpanIterator(this);
    }
    
    public SpanIterator getSpanIterator(int[] array) {
        SpanIterator spanIterator = this.getSpanIterator();
        spanIterator.intersectClipBox(array[0], array[1], array[2], array[3]);
        return spanIterator;
    }
    
    public SpanIterator filter(SpanIterator spanIterator) {
        if (this.bands == null) {
            spanIterator.intersectClipBox(this.lox, this.loy, this.hix, this.hiy);
            return spanIterator;
        }
        spanIterator = new RegionClipSpanIterator(this, spanIterator);
        return spanIterator;
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Region[[");
        sb.append(this.lox);
        sb.append(", ");
        sb.append(this.loy);
        sb.append(" => ");
        sb.append(this.hix);
        sb.append(", ");
        sb.append(this.hiy);
        sb.append("]");
        if (this.bands != null) {
            int i = 0;
            while (i < this.endIndex) {
                sb.append("y{");
                sb.append(this.bands[i++]);
                sb.append(",");
                sb.append(this.bands[i++]);
                sb.append("}[");
                while (i < i + this.bands[i++] * 2) {
                    sb.append("x(");
                    sb.append(this.bands[i++]);
                    sb.append(", ");
                    sb.append(this.bands[i++]);
                    sb.append(")");
                }
                sb.append("]");
            }
        }
        sb.append("]");
        return sb.toString();
    }
    
    @Override
    public int hashCode() {
        return this.isEmpty() ? 0 : (this.lox * 3 + this.loy * 5 + this.hix * 7 + this.hiy * 9);
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof Region)) {
            return false;
        }
        Region region = (Region)o;
        if (this.isEmpty()) {
            return region.isEmpty();
        }
        if (region.isEmpty()) {
            return false;
        }
        if (region.lox != this.lox || region.loy != this.loy || region.hix != this.hix || region.hiy != this.hiy) {
            return false;
        }
        if (this.bands == null) {
            return region.bands == null;
        }
        if (region.bands == null) {
            return false;
        }
        if (this.endIndex != region.endIndex) {
            return false;
        }
        int[] bands = this.bands;
        int[] bands2 = region.bands;
        int i = 0;
        while (i < this.endIndex) {
            if (bands[i] != bands2[i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    static {
        EMPTY_REGION = new Region.ImmutableRegion(0, 0, 0, 0);
        WHOLE_REGION = new Region.ImmutableRegion(Integer.MIN_VALUE, Integer.MIN_VALUE, Integer.MAX_VALUE, Integer.MAX_VALUE);
        initIDs();
    }
    
    private static final class ImmutableRegion extends Region
    {
        protected ImmutableRegion(int n, int n2, int n3, int n4) {
            super(n, n2, n3, n4);
        }
        
        @Override
        public void appendSpans(SpanIterator spanIterator) {
        }
        
        @Override
        public void setOutputArea(Rectangle rectangle) {
        }
        
        @Override
        public void setOutputAreaXYWH(int n, int n2, int n3, int n4) {
        }
        
        @Override
        public void setOutputArea(int[] array) {
        }
        
        @Override
        public void setOutputAreaXYXY(int n, int n2, int n3, int n4) {
        }
    }
}
